// Code generated by "stringer -type TokenType"; DO NOT EDIT.

package chroma

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[Background - -1]
	_ = x[LineNumbers - -2]
	_ = x[LineNumbersTable - -3]
	_ = x[LineHighlight - -4]
	_ = x[LineTable - -5]
	_ = x[LineTableTD - -6]
	_ = x[Error - -7]
	_ = x[Other - -8]
	_ = x[None - -9]
	_ = x[EOFType-0]
	_ = x[Keyword-1000]
	_ = x[KeywordConstant-1001]
	_ = x[KeywordDeclaration-1002]
	_ = x[KeywordNamespace-1003]
	_ = x[KeywordPseudo-1004]
	_ = x[KeywordReserved-1005]
	_ = x[KeywordType-1006]
	_ = x[Name-2000]
	_ = x[NameAttribute-2001]
	_ = x[NameBuiltin-2002]
	_ = x[NameBuiltinPseudo-2003]
	_ = x[NameClass-2004]
	_ = x[NameConstant-2005]
	_ = x[NameDecorator-2006]
	_ = x[NameEntity-2007]
	_ = x[NameException-2008]
	_ = x[NameFunction-2009]
	_ = x[NameFunctionMagic-2010]
	_ = x[NameKeyword-2011]
	_ = x[NameLabel-2012]
	_ = x[NameNamespace-2013]
	_ = x[NameOperator-2014]
	_ = x[NameOther-2015]
	_ = x[NamePseudo-2016]
	_ = x[NameProperty-2017]
	_ = x[NameTag-2018]
	_ = x[NameVariable-2019]
	_ = x[NameVariableAnonymous-2020]
	_ = x[NameVariableClass-2021]
	_ = x[NameVariableGlobal-2022]
	_ = x[NameVariableInstance-2023]
	_ = x[NameVariableMagic-2024]
	_ = x[Literal-3000]
	_ = x[LiteralDate-3001]
	_ = x[LiteralOther-3002]
	_ = x[LiteralString-3100]
	_ = x[LiteralStringAffix-3101]
	_ = x[LiteralStringAtom-3102]
	_ = x[LiteralStringBacktick-3103]
	_ = x[LiteralStringBoolean-3104]
	_ = x[LiteralStringChar-3105]
	_ = x[LiteralStringDelimiter-3106]
	_ = x[LiteralStringDoc-3107]
	_ = x[LiteralStringDouble-3108]
	_ = x[LiteralStringEscape-3109]
	_ = x[LiteralStringHeredoc-3110]
	_ = x[LiteralStringInterpol-3111]
	_ = x[LiteralStringName-3112]
	_ = x[LiteralStringOther-3113]
	_ = x[LiteralStringRegex-3114]
	_ = x[LiteralStringSingle-3115]
	_ = x[LiteralStringSymbol-3116]
	_ = x[LiteralNumber-3200]
	_ = x[LiteralNumberBin-3201]
	_ = x[LiteralNumberFloat-3202]
	_ = x[LiteralNumberHex-3203]
	_ = x[LiteralNumberInteger-3204]
	_ = x[LiteralNumberIntegerLong-3205]
	_ = x[LiteralNumberOct-3206]
	_ = x[Operator-4000]
	_ = x[OperatorWord-4001]
	_ = x[Punctuation-5000]
	_ = x[Comment-6000]
	_ = x[CommentHashbang-6001]
	_ = x[CommentMultiline-6002]
	_ = x[CommentSingle-6003]
	_ = x[CommentSpecial-6004]
	_ = x[CommentPreproc-6100]
	_ = x[CommentPreprocFile-6101]
	_ = x[Generic-7000]
	_ = x[GenericDeleted-7001]
	_ = x[GenericEmph-7002]
	_ = x[GenericError-7003]
	_ = x[GenericHeading-7004]
	_ = x[GenericInserted-7005]
	_ = x[GenericOutput-7006]
	_ = x[GenericPrompt-7007]
	_ = x[GenericStrong-7008]
	_ = x[GenericSubheading-7009]
	_ = x[GenericTraceback-7010]
	_ = x[GenericUnderline-7011]
	_ = x[Text-8000]
	_ = x[TextWhitespace-8001]
	_ = x[TextSymbol-8002]
	_ = x[TextPunctuation-8003]
}

const _TokenType_name = "NoneOtherErrorLineTableTDLineTableLineHighlightLineNumbersTableLineNumbersBackgroundEOFTypeKeywordKeywordConstantKeywordDeclarationKeywordNamespaceKeywordPseudoKeywordReservedKeywordTypeNameNameAttributeNameBuiltinNameBuiltinPseudoNameClassNameConstantNameDecoratorNameEntityNameExceptionNameFunctionNameFunctionMagicNameKeywordNameLabelNameNamespaceNameOperatorNameOtherNamePseudoNamePropertyNameTagNameVariableNameVariableAnonymousNameVariableClassNameVariableGlobalNameVariableInstanceNameVariableMagicLiteralLiteralDateLiteralOtherLiteralStringLiteralStringAffixLiteralStringAtomLiteralStringBacktickLiteralStringBooleanLiteralStringCharLiteralStringDelimiterLiteralStringDocLiteralStringDoubleLiteralStringEscapeLiteralStringHeredocLiteralStringInterpolLiteralStringNameLiteralStringOtherLiteralStringRegexLiteralStringSingleLiteralStringSymbolLiteralNumberLiteralNumberBinLiteralNumberFloatLiteralNumberHexLiteralNumberIntegerLiteralNumberIntegerLongLiteralNumberOctOperatorOperatorWordPunctuationCommentCommentHashbangCommentMultilineCommentSingleCommentSpecialCommentPreprocCommentPreprocFileGenericGenericDeletedGenericEmphGenericErrorGenericHeadingGenericInsertedGenericOutputGenericPromptGenericStrongGenericSubheadingGenericTracebackGenericUnderlineTextTextWhitespaceTextSymbolTextPunctuation"

var _TokenType_map = map[TokenType]string{
	-9:   _TokenType_name[0:4],
	-8:   _TokenType_name[4:9],
	-7:   _TokenType_name[9:14],
	-6:   _TokenType_name[14:25],
	-5:   _TokenType_name[25:34],
	-4:   _TokenType_name[34:47],
	-3:   _TokenType_name[47:63],
	-2:   _TokenType_name[63:74],
	-1:   _TokenType_name[74:84],
	0:    _TokenType_name[84:91],
	1000: _TokenType_name[91:98],
	1001: _TokenType_name[98:113],
	1002: _TokenType_name[113:131],
	1003: _TokenType_name[131:147],
	1004: _TokenType_name[147:160],
	1005: _TokenType_name[160:175],
	1006: _TokenType_name[175:186],
	2000: _TokenType_name[186:190],
	2001: _TokenType_name[190:203],
	2002: _TokenType_name[203:214],
	2003: _TokenType_name[214:231],
	2004: _TokenType_name[231:240],
	2005: _TokenType_name[240:252],
	2006: _TokenType_name[252:265],
	2007: _TokenType_name[265:275],
	2008: _TokenType_name[275:288],
	2009: _TokenType_name[288:300],
	2010: _TokenType_name[300:317],
	2011: _TokenType_name[317:328],
	2012: _TokenType_name[328:337],
	2013: _TokenType_name[337:350],
	2014: _TokenType_name[350:362],
	2015: _TokenType_name[362:371],
	2016: _TokenType_name[371:381],
	2017: _TokenType_name[381:393],
	2018: _TokenType_name[393:400],
	2019: _TokenType_name[400:412],
	2020: _TokenType_name[412:433],
	2021: _TokenType_name[433:450],
	2022: _TokenType_name[450:468],
	2023: _TokenType_name[468:488],
	2024: _TokenType_name[488:505],
	3000: _TokenType_name[505:512],
	3001: _TokenType_name[512:523],
	3002: _TokenType_name[523:535],
	3100: _TokenType_name[535:548],
	3101: _TokenType_name[548:566],
	3102: _TokenType_name[566:583],
	3103: _TokenType_name[583:604],
	3104: _TokenType_name[604:624],
	3105: _TokenType_name[624:641],
	3106: _TokenType_name[641:663],
	3107: _TokenType_name[663:679],
	3108: _TokenType_name[679:698],
	3109: _TokenType_name[698:717],
	3110: _TokenType_name[717:737],
	3111: _TokenType_name[737:758],
	3112: _TokenType_name[758:775],
	3113: _TokenType_name[775:793],
	3114: _TokenType_name[793:811],
	3115: _TokenType_name[811:830],
	3116: _TokenType_name[830:849],
	3200: _TokenType_name[849:862],
	3201: _TokenType_name[862:878],
	3202: _TokenType_name[878:896],
	3203: _TokenType_name[896:912],
	3204: _TokenType_name[912:932],
	3205: _TokenType_name[932:956],
	3206: _TokenType_name[956:972],
	4000: _TokenType_name[972:980],
	4001: _TokenType_name[980:992],
	5000: _TokenType_name[992:1003],
	6000: _TokenType_name[1003:1010],
	6001: _TokenType_name[1010:1025],
	6002: _TokenType_name[1025:1041],
	6003: _TokenType_name[1041:1054],
	6004: _TokenType_name[1054:1068],
	6100: _TokenType_name[1068:1082],
	6101: _TokenType_name[1082:1100],
	7000: _TokenType_name[1100:1107],
	7001: _TokenType_name[1107:1121],
	7002: _TokenType_name[1121:1132],
	7003: _TokenType_name[1132:1144],
	7004: _TokenType_name[1144:1158],
	7005: _TokenType_name[1158:1173],
	7006: _TokenType_name[1173:1186],
	7007: _TokenType_name[1186:1199],
	7008: _TokenType_name[1199:1212],
	7009: _TokenType_name[1212:1229],
	7010: _TokenType_name[1229:1245],
	7011: _TokenType_name[1245:1261],
	8000: _TokenType_name[1261:1265],
	8001: _TokenType_name[1265:1279],
	8002: _TokenType_name[1279:1289],
	8003: _TokenType_name[1289:1304],
}

func (i TokenType) String() string {
	if str, ok := _TokenType_map[i]; ok {
		return str
	}
	return "TokenType(" + strconv.FormatInt(int64(i), 10) + ")"
}
